import pytest

from muck_out.process import normalize_object
from muck_out.types import ObjectStub


@pytest.fixture
def sample_object():
    return {
        "@context": [
            "https://www.w3.org/ns/activitystreams",
            {"toot": "http://joinmastodon.org/ns#", "Emoji": "toot:Emoji"},
        ],
        "id": "https://activitypub.space/post/99",
        "type": "Article",
        "to": [
            "https://www.w3.org/ns/activitystreams#Public",
            "https://activitypub.space/category/5",
        ],
        "cc": [
            "https://activitypub.space/uid/1/followers",
            "https://piefed.social/u/rimu",
            "https://community.nodebb.org/uid/2",
            "https://indieweb.social/users/phillycodehound",
            "https://mitra.social/users/silverpill",
            "https://community.nodebb.org/uid/2/followers",
            "https://indieweb.social/users/phillycodehound/followers",
            "https://mitra.social/users/silverpill/followers",
        ],
        "inReplyTo": None,
        "published": "2025-09-06T20:09:12.263Z",
        "updated": None,
        "url": "https://activitypub.space/post/99",
        "attributedTo": "https://activitypub.space/uid/1",
        "context": "https://activitypub.space/topic/18",
        "audience": "https://activitypub.space/category/5",
        "summary": " <p>Hey <a href=\"https://piefed.social/u/rimu\">@<bdi>rimu@piefed.social</bdi></a> question to you about post removal...</p> <p>If a remote user posts to a local community, and the local mod deletes it (let's say it's spam of off topic), does the local community federate a delete out?</p> <p>Technically you're not <strong>deleting</strong> the content, just removing it from the community.</p> <p>Is there a different action Piefed takes?</p>",
        "name": "Topic removal from a category/community",
        "preview": {
            "type": "Note",
            "attributedTo": "https://activitypub.space/uid/1",
            "content": "<p>Hey <a href=\"https://piefed.social/u/rimu\">@<bdi>rimu@piefed.social</bdi></a> question to you about post removal...</p>\n<p>If a remote user posts to a local community, and the local mod deletes it (let's say it's spam of off topic), does the local community federate a delete out?</p>\n<p>Technically you're not <strong>deleting</strong> the content, just removing it from the community.</p>\n<p>Is there a different action Piefed takes?</p>\n",
            "published": "2025-09-06T20:09:12.263Z",
            "attachment": [],
        },
        "content": "<p>Hey <a href=\"https://piefed.social/u/rimu\">@<bdi>rimu@piefed.social</bdi></a> question to you about post removal...</p>\n<p>If a remote user posts to a local community, and the local mod deletes it (let's say it's spam of off topic), does the local community federate a delete out?</p>\n<p>Technically you're not <strong>deleting</strong> the content, just removing it from the community.</p>\n<p>Is there a different action Piefed takes?</p>\n",
        "source": {
            "content": "Hey [rimu@piefed.social](https://activitypub.space/user/rimu%40piefed.social) question to you about post removal...\n\nIf a remote user posts to a local community, and the local mod deletes it (let's say it's spam of off topic), does the local community federate a delete out?\n\nTechnically you're not **deleting** the content, just removing it from the community.\n\nIs there a different action Piefed takes?",
            "mediaType": "text/markdown",
        },
        "tag": [
            {
                "type": "Hashtag",
                "href": "https://activitypub.space/tags/piefed",
                "name": "#piefed",
            },
            {
                "type": "Mention",
                "href": "https://piefed.social/u/rimu",
                "name": "@rimu@piefed.social",
            },
        ],
        "attachment": [],
        "replies": "https://activitypub.space/post/99/replies",
    }


def test_normalize_object(sample_object):
    result = normalize_object(sample_object)

    assert result.id == sample_object["id"]
    assert result.context == sample_object["context"]


def test_object_stub(sample_object):
    result = ObjectStub.model_validate(sample_object)

    assert result.id == sample_object["id"]
    assert result.context == sample_object["context"]



from datetime import datetime
from pydantic import BaseModel, Field

from muck_out.validators import HtmlStringOrNone, IdFieldOrNone


class CommonObject(BaseModel):
    updated: datetime | None = Field(
        default=None,
        description="Moment of this object being updated",
    )
    summary: HtmlStringOrNone = Field(
        default=None,
        description="The summary of the object",
    )
    name: HtmlStringOrNone = Field(
        default=None,
        description="The name of the object",
    )
    inReplyTo: IdFieldOrNone = Field(
        None,
        description="The object being replied to. Currently a string. Not sure if this is what I want.",
    )
    context: IdFieldOrNone = Field(None, description="The context of the object")



from muck_out.types import Actor
from .types import ActorHeaderInfo

__all__ = ["ActorHeaderInfo", "actor_to_header_info"]


def actor_to_header_info(actor: Actor) -> ActorHeaderInfo:
    """Turns an [Actor][muck_out.types.Actor] object into a reduced version
    suitable to display in a header lien for this actor.

    ```python
    >>> from muck_out.testing.examples import actor
    >>> result = actor_to_header_info(actor)
    >>> print(result.model_dump_json(indent=2))
    {
      "id": "http://abel/actor/AFKb0cQunSBv1fC7sWbQYg",
      "avatarUrl": "https://dev.bovine.social/assets/bull-horns.png",
      "name": "The kitty",
      "identifier": "acct:kitty@abel",
      "htmlUrl": "http://abel/@kitty"
    }

    ```
    """
    avatar_url = actor.icon.get("url") if actor.icon else None

    html_url = None
    for url in actor.url:
        if url.get("mediaType") == "text/html":
            html_url = url.get("href")
            if html_url:
                break
    if not html_url and len(actor.url) > 0:
        html_url = actor.url[0].get("href")

    return ActorHeaderInfo(
        id=actor.id,
        name=actor.name,
        identifier=actor.identifiers[0],
        avatarUrl=avatar_url,
        htmlUrl=html_url,
    )



from pydantic import BaseModel, ConfigDict, Field

from muck_out.validators import HtmlStringOrNone


class ActorHeaderInfo(BaseModel):
    """Derived information suitable to be displayed in a header line."""

    model_config = ConfigDict(
        serialize_by_alias=True,
        extra="forbid",
        field_title_generator=lambda field_name, field_info: "",
    )

    id: str = Field(
        examples=["https://actor.example/alice/some/id"],
        description="id of the actor, can be assumed to be globally unique. Some activities such as a Follow request will require an id to be valid. Servers may assume an id to be required. As assigning an id is 'trivial', one should assign one.",
    )

    avatar_url: str | None = Field(
        default=None,
        examples=["https://actor.example/static/alice-avatar.png"],
        description="The url of the avatar to use for the actor",
        alias="avatarUrl",
    )

    name: HtmlStringOrNone = Field(
        default=None,
        examples=["Alice"],
        description="Display name of the actor",
    )

    identifier: str = Field(
        examples=["acct:alice@actor.example"],
        description="Identifier to display for the actor",
    )

    html_url: str | None = Field(
        default=None,
        examples=["https://actor.example/@alice"],
        description="Location of a html representation of the actor",
        alias="htmlUrl",
    )



import pytest
from . import object_stub


@pytest.mark.parametrize(
    "data", [{"content": "text"}, {"content": ["text"]}, {"contentMap": {"en": "text"}}]
)
def test_processes_content(data):
    result = object_stub(data)
    assert result.content == "text"
